@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -53,19 +53,27 @@ typedef struct Line
 #if JED_HAS_LINE_ATTRIBUTES
    /* The first 4 bits of the flags field represent a 4 bit integer that
     * is used by the syntax parsing routines.  The next 4 bits
-    * control the various flags.  Finally the last 8 bits specify the
-    * color of the line.
+    * control the various flags.  The last 8 bits specify the
+    * color of the line.  
+    *
+    * Finally, assuming at least 32 bit integers, the last 16 bits may be
+    * used for the syntax state of the line. For instance, if the line is in
+    * a string, the string character may be specified by the some of the bits,
+    * which is useful if there are more than one.
     */
    unsigned int flags;
 #endif
 #define JED_LINE_IN_COMMENT		0x0001
 #define JED_LINE_IN_STRING		0x0002
+
 #define JED_LINE_HAS_EOL_COMMENT	0x0008
 #define JED_LINE_SYNTAX_BITS		0x000F
    
 #define JED_LINE_HIDDEN			0x0010
 #define JED_LINE_IS_READONLY		0x0020
    
+#define JED_LINE_COLOR_BITS		0xFF00
+
 } Line;
 
 /* This is the price we pay for a linked list approach.  With straight
@@ -156,7 +164,9 @@ struct _Buffer
    int hits;			       /* number of hits on buffer since 
 					* last autosave.  A hit is the number
 					* of times the buffer was hit on at top level  */
+#if 0
    unsigned long m_time;	       /* time when buffer first modified */
+#endif
    unsigned long c_time;	       /* time when buffer first created or  */
    /* when file visited */
    Undo_Type *undo;		       /* pointer to undo ring */
@@ -243,6 +253,7 @@ extern char Default_Status_Line[80];
 #ifndef VMS
 #define MAP_CR_TO_NL_FLAG 0x1000
 #endif
+#define BUFFER_NON_LOCKING	0x8000
 
 extern char *Read_Only_Error;
 extern char *Line_Read_Only_Error;
