@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -224,7 +224,7 @@ static int sys_open (char *file, int acces) /*{{{*/
    strcpy(neew, file);
    p = neew; while (*p) if (*p == ';') *p = 0; else p++;
    
-   switch (access)
+   switch (acces)
      {
       case _JED_OPEN_READ:
 	return open(file, O_RDONLY, "ctx=rec","mbf=8","mbc=32","rop=RAH","shr=upi,get,put");
@@ -330,36 +330,47 @@ void set_file_modes (void) /*{{{*/
 #endif
 
 #ifdef W_OK
-#ifdef __GO32__
-# define access i386_access
-#endif
-	char name [JED_MAX_PATH_LEN + 256];
-#ifndef VMS
+# ifdef __GO32__
+#  define access i386_access
+# endif
+	char *file = jed_dir_file_merge (CBuf->dir, CBuf->file);
+	if (file != NULL)
 	  {
-	     /* COmment:  I need a more generic call to contruct a directory
-	      * name.  site.sl already does this properly.
-	      */
-	     int n;
-	     strcpy (name, CBuf->dir);
-	     n = strlen (name);
-	     if (n >
-#ifdef IBMPC_SYSTEM
-		 3		       /* allow C:/file */
-#else
-		 1		       /* allow /file */
-#endif
-		 )
+	     if (0 == access(file, F_OK))
 	       {
-		  name[n - 1] = 0;     /* knock off slash */
-		  if (!access (name, F_OK) && (access (name, W_OK)))
+		  if (-1 == access(file, W_OK))
 		    CBuf->flags |= READ_ONLY;
 	       }
+	     else 
+	       {
+		  /* file does not exist.  Can we write to the directory? */
+		  /* knock of slash since some primitive systems cannot handle the 
+		   * final slash in a path name. 
+		   */
+		  char *dir;
+#ifdef IBMPC_SYSTEM
+		  dir = extract_file (file);
+		  *dir = 0;
+		  if (strlen (file) > 3)   /* allow C:/, but not C:/xx/ */
+		    *--dir = 0;
+		  dir = file;
+#else
+		  dir = CBuf->dir;
+#endif
+		  if ((0 == access (dir, F_OK))
+#ifdef VMS
+		      && (-1 == access (dir, X_OK))
+#else
+		      && (-1 == access (dir, W_OK))
+#endif
+		      )
+		    CBuf->flags |= READ_ONLY;
+	       }
+
+	     SLfree (file);
 	  }
-#endif  /* NOT VMS */
-	sprintf (name, "%s%s", CBuf->dir, CBuf->file);
-	if (!access(name, F_OK) && access(name, W_OK))
-	  CBuf->flags |= READ_ONLY;
-#endif /* W_OK */
+	     
+#endif				       /* W_OK */
 	CBuf->flags |= AUTO_SAVE_BUFFER;
 	CBuf->hits = 0;
 	type = file_type(CBuf->file);
@@ -393,6 +404,7 @@ int read_file(char *file) /*{{{*/
      return -1;
    
    if (status > 0)
+     /* FIXME!! Is this true?  What if no lines have been read. */
      n = Max_LineNum;
    else
      {
@@ -1115,7 +1127,14 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
      }
 #endif				       /* NOT VMS */
 
-   if (-1 != (n = write_file_internal (neew, _JED_OPEN_WRITE)))
+   n = write_file_internal (neew, _JED_OPEN_WRITE);
+   /* This is for NFS time problems.  Even if the write failed, modify the 
+    * buffer's ctime because otherwise what is on the disk (a partial file) 
+    * will appear newer than the buffer.
+    */
+   CBuf->c_time = sys_file_mod_time(neew);
+
+   if (n != -1)
      {
 	if (*autosave_file)
 	  (void) sys_delete_file (autosave_file);
@@ -1131,8 +1150,6 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
 	     WriteEAs (neew, EAs);
 #endif
 	  }
-	/* This is for NFS time problems */
-	CBuf->c_time = sys_file_mod_time(neew);
 	
 	/* Since we wrote the buffer to the file, it is not modified. */
 	if (
@@ -1146,12 +1163,11 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
 	  CBuf->flags &= ~FILE_MODIFIED;
 	
 	mark_buffer_modified (CBuf, 0, 1);
-	/* CBuf->c_time = sys_time(); */
      }
 #ifndef VMS
    /* error -- put it back */
-   else if (backup_went_ok) perform_backup (old, neew, 1); 
-   
+   else if (backup_went_ok) perform_backup (old, neew, 1);
+
    if (old != NULL) SLang_free_slstring (old);
 #endif
    return(n);
@@ -1289,7 +1305,8 @@ char *jed_expand_link(char *f) /*{{{*/
 {
    char work[JED_MAX_PATH_LEN], lnk[JED_MAX_PATH_LEN];
    char *d = jed_expand_filename (f);
-   
+   unsigned int max_num = 20;
+
    while ((d != NULL) && is_link(d, lnk))
      {
 	char *w, *w1, ch;
@@ -1315,6 +1332,12 @@ char *jed_expand_link(char *f) /*{{{*/
 	*w = 0;
 	
 	SLfree (d);
+	max_num--;
+	if (max_num == 0)
+	  {
+	     jed_verror ("possible circular-link detected");
+	     return NULL;
+	  }
 	d = jed_expand_filename(work);
      }
    
