@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -142,7 +142,13 @@ int _jed_is_eol_comment_start (Syntax_Table_Type *table, Line *l,
 		  if (wsc)
 		    {
 		       if ((ch1 != ' ') && (ch1 != '\n') && (ch1 != '\t'))
-			 continue;
+			 {
+			    /* This piece of code permits: #### */
+			    if ((ch1 != *p)
+				|| (table->char_syntax[ch1] & WORD_SYNTAX)
+				|| (p1 != p + 1))
+			      continue;
+			 }
 		    }
 		  else if (table->char_syntax[ch1] & WORD_SYNTAX)
 		    continue;
@@ -155,7 +161,13 @@ int _jed_is_eol_comment_start (Syntax_Table_Type *table, Line *l,
 		  if (wsc)
 		    {
 		       if ((ch1 != ' ') && (ch1 != '\t'))
-			 continue;
+			 {
+			    /* This piece of code permits: #### */
+			    if ((ch1 != *p)
+				|| (table->char_syntax[ch1] & WORD_SYNTAX)
+				|| (p1 != p + 1))
+			      continue;
+			 }
 		    }
 		  else if (table->char_syntax[ch1] & WORD_SYNTAX)
 		    continue;
@@ -975,7 +987,7 @@ static int find_matching_delimiter_1 (unsigned char ch, int nlines)
      return backward_goto_match (nlines, ch);
 }
 
-int find_matching_delimiter (int *ch)
+static int find_matching_delimiter (int *ch)
 {
    return find_matching_delimiter_1 ((unsigned char) *ch, 5000);
 }
@@ -992,7 +1004,7 @@ int goto_match (void) /*{{{*/
 
 /*}}}*/
 
-int parse_to_point (void) /*{{{*/
+static int parse_to_point (void) /*{{{*/
 {
    int state;
 
@@ -1082,7 +1094,7 @@ void blink_match (void) /*{{{*/
 
 /*}}}*/
 
-Syntax_Table_Type *find_syntax_table (char *name, int err) /*{{{*/
+Syntax_Table_Type *jed_find_syntax_table (char *name, int err) /*{{{*/
 {
    Syntax_Table_Type *table = Syntax_Tables;
    while (table != NULL)
@@ -1096,11 +1108,11 @@ Syntax_Table_Type *find_syntax_table (char *name, int err) /*{{{*/
 
 /*}}}*/
 
-void set_syntax_flags (char *name, int *flags) /*{{{*/
+static void set_syntax_flags (char *name, int *flags) /*{{{*/
 {
    Syntax_Table_Type *table;
 
-   table = find_syntax_table (name, 1);
+   table = jed_find_syntax_table (name, 1);
    if (table == NULL) return;
 
    table->flags |= *flags & 0xFF;
@@ -1108,7 +1120,7 @@ void set_syntax_flags (char *name, int *flags) /*{{{*/
 
 /*}}}*/
 
-void define_syntax (int *what, char *name) /*{{{*/
+static void define_syntax (int *what, char *name) /*{{{*/
 {
    Syntax_Table_Type *table;
    int c2;
@@ -1116,7 +1128,7 @@ void define_syntax (int *what, char *name) /*{{{*/
    char *s1 = NULL, *s2 = NULL;
    unsigned char lut[256], *s;
 
-   table = find_syntax_table (name, 1);
+   table = jed_find_syntax_table (name, 1);
    if (table == NULL) return;
 
    switch (*what)
@@ -1267,12 +1279,12 @@ void define_syntax (int *what, char *name) /*{{{*/
 
 /*}}}*/
 
-void jed_set_fortran_comment_style (char *table_name, char *str)
+static void set_fortran_comment_style (char *table_name, char *str)
 {
    Syntax_Table_Type *table;
    int reverse;
 
-   if (NULL == (table = find_syntax_table (table_name, 1)))
+   if (NULL == (table = jed_find_syntax_table (table_name, 1)))
      return;
    
    reverse = 0;
@@ -1284,18 +1296,23 @@ void jed_set_fortran_comment_style (char *table_name, char *str)
    SLmake_lut(table->fortran_comment_chars, (unsigned char *) str, reverse);
 }
 
-void use_syntax_table (char *s) /*{{{*/
+static void use_syntax_table (char *s) /*{{{*/
 {
    Syntax_Table_Type *table;
 
    if ((s == NULL) || (*s == 0))
-     table = Default_Syntax_Table;
+     {
+	s = "";
+	table = Default_Syntax_Table;
+     }
    else
      {
-	table = find_syntax_table (s, 1);
+	table = jed_find_syntax_table (s, 1);
 	if (table == NULL) return;
      }
    CBuf->syntax_table = table;
+
+   (void) SLang_run_hooks ("use_syntax_table_hook", 1, s);
 }
 
 /*}}}*/
@@ -1334,11 +1351,12 @@ static Syntax_Table_Type *allocate_syntax_table (char *name)
    return table;
 }
 
-void create_syntax_table (char *name) /*{{{*/
+
+static void create_syntax_table (char *name) /*{{{*/
 {
    Syntax_Table_Type *table;
 
-   if (NULL != (table = find_syntax_table (name, 0)))
+   if (NULL != (table = jed_find_syntax_table (name, 0)))
      {
 	clear_syntax_table (table);
 	return;
@@ -1351,7 +1369,6 @@ void create_syntax_table (char *name) /*{{{*/
    Syntax_Tables = table;
 }
 
-/*}}}*/
 
 void init_syntax_tables (void) /*{{{*/
 {
@@ -1374,13 +1391,13 @@ void init_syntax_tables (void) /*{{{*/
 
 /*}}}*/
 
-void define_keywords (char *name, char *kwords, int *lenp, int *tbl_nump) /*{{{*/
+static void define_keywords (char *name, char *kwords, int *lenp, int *tbl_nump) /*{{{*/
 {
    char *kw;
    int len;
    int kwlen;
    unsigned int table_number = (unsigned int) *tbl_nump;
-   Syntax_Table_Type *table = find_syntax_table (name, 1);
+   Syntax_Table_Type *table = jed_find_syntax_table (name, 1);
 
    if (table == NULL) return;
 
@@ -1470,6 +1487,8 @@ void jed_syntax_parse_buffer (int do_all)
    int is_narrow;
    Line *l;
    Syntax_Table_Type *table;
+   unsigned int undo_bit = 0;
+
 
    table = CBuf->syntax_table;
    if (table == NULL)
@@ -1491,6 +1510,10 @@ void jed_syntax_parse_buffer (int do_all)
    is_narrow = (CBuf->narrow != NULL);
    if (is_narrow)
      {
+	/* yuk--- a major hack: turn off undo temporally */
+	undo_bit = CBuf->flags & UNDO_ENABLED;
+	CBuf->flags &= ~UNDO_ENABLED;
+
 	jed_push_narrow ();
 	jed_widen_whole_buffer (CBuf);
      }
@@ -1510,9 +1533,34 @@ void jed_syntax_parse_buffer (int do_all)
    syntax_parse_lines (table, l, 1 + (unsigned int) (max_line_num - min_line_num));
 
    if (is_narrow)
-     jed_pop_narrow ();
+     {
+	jed_pop_narrow ();
+	CBuf->flags |= undo_bit;       /* hack */
+     }
 
    CBuf->min_unparsed_line_num = CBuf->max_unparsed_line_num = 0;
 }
 #endif
 
+static SLang_Intrin_Fun_Type Intrinsics [] = 
+{
+   MAKE_INTRINSIC("parse_to_point", parse_to_point, INT_TYPE, 0),
+   MAKE_INTRINSIC_SI("set_syntax_flags", set_syntax_flags, VOID_TYPE),
+   MAKE_INTRINSIC_IS("define_syntax", define_syntax, VOID_TYPE),
+   MAKE_INTRINSIC_S("use_syntax_table", use_syntax_table, VOID_TYPE),
+   MAKE_INTRINSIC_S("create_syntax_table", create_syntax_table, VOID_TYPE),
+   MAKE_INTRINSIC_4("define_keywords_n", define_keywords, VOID_TYPE, STRING_TYPE, STRING_TYPE, INT_TYPE, INT_TYPE),
+   MAKE_INTRINSIC_SS("set_fortran_comment_chars", set_fortran_comment_style, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("find_matching_delimiter", find_matching_delimiter, INT_TYPE),
+   SLANG_END_INTRIN_FUN_TABLE
+};
+
+int jed_init_syntax (void)
+{
+#if JED_HAS_DFA_SYNTAX
+   if (-1 == jed_init_dfa_syntax ())
+     return -1;
+#endif
+
+   return SLadd_intrin_fun_table (Intrinsics, NULL);
+}
