@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -61,6 +61,8 @@
 #include "hooks.h"
 #include "undo.h"
 #include "menu.h"
+#include "userinfo.h"
+#include "indent.h"
 
 #ifdef __WIN32__
 # include "win32.h"
@@ -93,8 +95,6 @@ static int Buf_List_Len;
 Buffer *Buf_Context;
 
 static char *Expect_File_Error = "Expecting filename.";
-static char *Keymap_Error = "Unknown_Keymap";
-
 
 static int next_bufflist(char *buf) /*{{{*/
 {
@@ -310,72 +310,106 @@ int kill_buffer_cmd(char *name) /*{{{*/
 
 /*}}}*/
 
-int write_buffer_cmd(char *filestr) /*{{{*/
+int jed_save_buffer_to_file (char *dir, char *file)
 {
-#ifdef VMS
-   char *ff;
-#endif
-   char dir[JED_MAX_PATH_LEN], *f, file[JED_MAX_PATH_LEN];
-   int n;
-   
-#ifdef REAL_UNIX_SYSTEM
-   f = expand_link(filestr);
-#else
-   f = expand_filename(filestr);
-#endif
-   
-   strncpy (file, f, JED_MAX_PATH_LEN); file[JED_MAX_PATH_LEN - 1] = 0;
-   f = extract_file(file);
-   
-   if ((*f == 0) && (*CBuf->file == 0))
+   int status;
+   char *dirfile;
+
+   status = -1;
+
+   if ((file == NULL) || (*file == 0))
      {
-	msg_error("Filename Required.");
-	return(0);
-     }
-   
-   n = (int) (f - file);
-   SLMEMCPY((char *) dir, (char *) file, n);
-   dir[n] = 0;
-   
-   if (*f == 0) 
-     {
-	safe_strcpy (file, CBuf->file, sizeof (file));
-	f = file;
+	file = CBuf->file;
+	if (*file == 0)
+	  {	     
+	     msg_error ("Filename Required.");
+	     return -1;
+	  }
      }
 
-   if (file_changed_on_disk (CBuf, dir_file_merge (dir, f)))
+   if (NULL == (dirfile = jed_dir_file_merge (dir, file)))
+     return -1;
+
+   if (file_changed_on_disk (CBuf, dirfile) > 0)
      {
 	if (0 == jed_get_yes_no("File changed on disk.  Save anyway?"))
-	  return 0;
-     }
-	  
-   n = write_file_with_backup (dir, f);
-   if (n >= 0)
-     {
-#ifdef VMS
-	ff = f; while (*ff) if (*ff == ';') *ff = 0; else ff++;
-#endif
-	if (Batch != 2)
 	  {
-	     jed_vmessage (0, "Wrote %d lines to %s%s", n, dir, f);
+	     SLfree (dirfile);
+	     return -1;
 	  }
-	CBuf->flags &= ~BUFFER_MODIFIED;
-	CBuf->flags |= AUTO_SAVE_BUFFER;
-	CBuf->hits = 0;
-#ifdef UNDO_HAS_REDO
-	update_undo_unchanged ();
-#endif
-	visit_file(dir, f);
-	return(1);
      }
-   else
+
+   if (-1 == jed_va_run_hooks ("_jed_save_buffer_before_hooks",
+			       JED_HOOKS_RUN_ALL, 1, dirfile))
      {
-	jed_verror ("Error writing file %s%s", dir, f);
-	return 0;
+	auto_save_buffer (CBuf);
+	SLfree (dirfile);
+	return -1;
      }
+
+   if (((status = write_file_with_backup (dir, file)) >= 0)
+       && (Batch != 2))
+     jed_vmessage (0, "Wrote %d lines to %s", status, dirfile);
+
+   if (status < 0)
+     {
+	jed_verror ("Error writing file %s", dirfile);
+	SLfree (dirfile);
+	return -1;
+     }
+
+	
+   CBuf->flags |= AUTO_SAVE_BUFFER;
+   CBuf->hits = 0;
+
+#ifdef UNDO_HAS_REDO
+   update_undo_unchanged ();
+#endif
+   visit_file(dir, file);
+
+   if (-1 == jed_va_run_hooks ("_jed_save_buffer_after_hooks", JED_HOOKS_RUN_ALL,
+			       1, dirfile))
+     {
+	SLfree (dirfile);
+	return -1;
+     }
+   
+   SLfree (dirfile);
+   return 0;
 }
 
-/*}}}*/
+
+int write_buffer_cmd (char *filestr) 
+{
+   char *dirfile, *dir, *file;
+   int status;
+
+   dirfile = jed_get_canonical_pathname (filestr);
+
+   if (dirfile == NULL)
+     return -1;
+   
+   if (-1 == jed_dirfile_to_dir_file (dirfile, &dir, &file))
+     {
+	SLfree (dirfile);
+	return -1;
+     }
+
+   status = jed_save_buffer_to_file (dir, file);
+
+   SLfree (dirfile);
+   SLfree (dir);
+   SLfree (file);
+
+   if (status == -1)
+     return status;
+
+   /* This function is an intrinsic and is documented to return the 
+    * number of lines written out.
+    * 
+    */
+   return 0;
+}
 
 #if defined(__BORLANDC__) && !defined(MSWINDOWS)
 int show_memory() /*{{{*/
@@ -493,6 +527,20 @@ int set_buffer(char *name) /*{{{*/
 int jed_get_mini_response (char *s)
 {
    int ch;
+   int len;
+
+   if (Batch)
+     {
+	if (EOF == fputs (s, stdout))
+	  exit_error ("Failed to write to stdout", 0);
+	
+	return jed_getkey ();
+     }
+	    
+   len = strlen (s);
+   if (len > Jed_Num_Screen_Cols)
+     s += (len - Jed_Num_Screen_Cols);
+
    MiniBuf_Get_Response_String = s;
    update (NULL, 0, 0, 0);
    ch = jed_getkey ();
@@ -531,7 +579,7 @@ int jed_get_y_n (char *question)
      return 1;
    if ((ans == 'n') || (ans == 'N'))
      return 0;
-   if (SLKeyBoard_Quit) 
+   if (SLKeyBoard_Quit || (SLang_Error == SL_USER_BREAK))
      return -1;
    jed_beep ();
    flush_input ();
@@ -597,79 +645,113 @@ int jed_get_yes_no (char *question) /*{{{*/
 
 int find_file_cmd (char *filestr)       /* filestr is const ! */ /*{{{*/
 {
-   char *f, *file, filebuf[JED_MAX_PATH_LEN];
    Buffer *buf;
-   int n;
-   
+   int status;
+   char *dirfile, *dir, *file;
+
+   /* If this hook returns 1, then the file is considered to be found. */
+   if (1 == jed_va_run_hooks ("_jed_find_file_before_hooks", JED_HOOKS_RUN_UNTIL_NON_0,
+			      1, filestr))
+     return 0;
+
 #ifdef REAL_UNIX_SYSTEM
-   file = expand_link(filestr);
+   dirfile = jed_expand_link(filestr);
 #else
-   file = expand_filename(filestr);
+   dirfile = jed_expand_filename(filestr);
 #endif
-   
-   safe_strcpy(filebuf, file, sizeof (filebuf));
-   file = filebuf;
-   f = extract_file(file);
-   if ((*file == 0) || (*f == 0))
+
+   if (dirfile == NULL)
+     return -1;
+
+   if (-1 == jed_dirfile_to_dir_file (dirfile, &dir, &file))
      {
-	msg_error(Expect_File_Error);
-	return(0);
+	SLfree (dirfile);
+	return -1;
      }
-   
+
+   status = -1;
+
+   if (*file == 0)
+     {
+	jed_verror (Expect_File_Error);
+	goto free_and_return;
+     }
+
    check_buffers ();
    
    /* search for the file in current buffer list */
    
-   if (NULL != (buf = find_file_buffer(file)))
+   if (NULL != (buf = find_file_buffer (dirfile)))
      {
-	if (file_changed_on_disk (buf, file))
+	if ((file_changed_on_disk (buf, dirfile) > 0)
+	    && (1 == jed_get_yes_no("File changed on disk.  Read From Disk")))
 	  {
-	     if (jed_get_yes_no("File changed on disk.  Read From Disk"))
-	       {
-		  if (*Error_Buffer) return(1);
-		  n = (int) buf->linenum;
-		  buf->flags  &= ~BUFFER_MODIFIED;
-		  kill_buffer_cmd(buf->name);
-		  find_file_cmd(file);
-		  goto_line(&n);
-		  return(1);
-	       }
+	     int n = (int) buf->linenum;
+	     mark_buffer_modified (buf, 0, 1);
+	     kill_buffer_cmd (buf->name);
+	     status = find_file_cmd (dirfile);
+	     goto_line (&n);
+	     goto free_and_return;
 	  }
 	
+	if (SLang_Error)
+	  goto free_and_return;
+
+	status = 1;
 	switch_to_buffer (buf);
-	return(1);
+	goto free_and_return;
      }
-   
-   buf = make_buffer (NULL, file, NULL);
+
+   /* Make a buffer but do not attach it to a file yet. */
+   buf = make_buffer (file, dir, NULL);
    switch_to_buffer(buf);
-   
-   n = read_file(file);
-   CLine = CBuf->beg;
-   Point = 0;
-   LineNum = 1;
-   if (CLine == NULL) make_line(25);
-   set_file_modes();
-   CBuf->flags |= UNDO_ENABLED;
-   
-   switch(n)
+
+   status = read_file (dirfile);
+	
+   switch (status)
      {
-      case -2: msg_error("File not readable."); break;
+      case -2:
+	jed_verror ("File %s not readable.", dirfile);
+	status = -1;
+	break;
+	     
+      case -1:
+	message ("New file.");
+	status = 0;
+	break;
+	     
       default:
-	if (Batch == 2) break;
-	if (*Message_Buffer) break;   /* autosave warning? */
-	if (n == -1) message("New file.");
-	else
-	  {
-	     jed_vmessage (0, "%d lines read", n);
-	  }
+	jed_vmessage (0, "%d lines read", status);
+	status = 1;
      }
-   
-   SLang_run_hooks("find_file_hook", 1, file);
-   return(1);
+
+   if (buf == CBuf)		       /* check this in case the hook switches buffers */
+     {
+	mark_buffer_modified (CBuf, 0, 0);
+
+	/* Now attach it to a file */
+	buffer_filename (CBuf, dir, file);
+	
+	(void) bob ();
+	set_file_modes();
+	CBuf->flags |= UNDO_ENABLED;
+     }
+
+   if (-1 == jed_va_run_hooks ("_jed_find_file_after_hooks", JED_HOOKS_RUN_ALL, 0))
+     status = -1;
+
+   free_and_return:
+
+   SLfree (dirfile);
+   SLfree (dir);
+   SLfree (file);
+
+   return status;
 }
 
 /*}}}*/
 
+
 int find_file_in_window(char *file) /*{{{*/
 {
    int ret;
@@ -690,6 +772,13 @@ static void create_minibuffer(void) /*{{{*/
    Window_Type *w;
    MiniBuffer = The_MiniBuffer;
    
+   /* We may get called before the keymap has been set up.  Make sure that
+    * we have a keymap.
+    */
+   if ((The_MiniBuffer->keymap == NULL)
+       || (The_MiniBuffer->keymap == Global_Map))
+     jed_setup_minibuffer_keymap ();
+
    /* I want to make Mini_Window->next = Current Window so that scroll other
       window routines will scroll it. */
    
@@ -704,6 +793,11 @@ static void create_minibuffer(void) /*{{{*/
    switch_to_buffer(MiniBuffer);
    MiniBuffer_Active = 1;
    erase_buffer ();
+
+#if JED_HAS_TTY_MENUS
+   jed_notify_menu_buffer_changed ();
+#endif
+
    /* allow kill region to kill to beginning of minibuffer. 
     * Suggested by stefen@uni-paderborn.de */
    push_mark ();
@@ -724,7 +818,7 @@ int exit_minibuffer() /*{{{*/
 	if (Last_Completion_Buffer != NULL)
 	  {
 	     pop_to_buffer (Completion_Buffer);
-	     CBuf->flags &= ~BUFFER_MODIFIED;
+	     mark_buffer_modified (CBuf, 0, 0);
 	     switch_to_buffer_cmd (Last_Completion_Buffer);
 	     kill_buffer_cmd (Completion_Buffer);
 	     touch_window_hard (JWindow, 0);
@@ -899,13 +993,15 @@ void read_object_with_completion(char *prompt, char *dflt, char *stuff, int *typ
 
 /*}}}*/
 
-int insert_file_cmd() /*{{{*/
+int insert_file_cmd (void) /*{{{*/
 {
    char *filebuf, *f, *file;
    
    CHECK_READ_ONLY
      
-     if (NULL == (filebuf = read_file_from_minibuffer("Insert file:", NULL))) return(0);
+     if (NULL == (filebuf = read_file_from_minibuffer("Insert file:", NULL)))
+       return -1;
+
    file = expand_filename(filebuf);
    SLfree (filebuf);
    f = extract_file(file);
@@ -923,25 +1019,25 @@ int insert_file_cmd() /*{{{*/
 
 int find_file (void) /*{{{*/
 {
-   char *tmp, *f;
-   char file [JED_MAX_PATH_LEN];
-   
-   if (NULL == (tmp = read_file_from_minibuffer("Find file:", (char *) NULL))) 
+   char *file, *f;
+   int status;
+
+   if (NULL == (file = read_file_from_minibuffer("Find file:", (char *) NULL))) 
      return 0;
    
-   safe_strcpy (file, tmp, sizeof (file));
-   SLfree(tmp);
-   
    f = extract_file (file);
    if (*f == 0)
      {
-	/* Use buffer filename as default. */
-	safe_strcpy (f, CBuf->file, 
-		     sizeof(file) - (unsigned int) (f - file));
+	char *dirfile = jed_dir_file_merge (file, CBuf->file);
+	SLfree (file);
+	if (dirfile == NULL)
+	  return -1;
+	file = dirfile;
      }
-   find_file_in_window (file);
 
-   return 0;
+   status = find_file_in_window (file);
+   SLfree (file);
+   return status;
 }
 
 /*}}}*/
@@ -951,7 +1047,7 @@ int write_buffer (void) /*{{{*/
    char *tmp;
    
    if (NULL == (tmp = read_file_from_minibuffer("Write to file:", (char *) NULL))) return(0);
-   write_buffer_cmd(tmp);
+   (void) write_buffer_cmd (tmp);
    SLfree(tmp);
    return(1);
 }
@@ -1013,7 +1109,10 @@ int kill_buffer (void) /*{{{*/
 	if ((b != NULL) && (b->subprocess))
 	  {
 	     if (0 == jed_get_yes_no("Buffer has a subprocess attached.  Delete anyway"))
-	       return 0;
+	       {
+		  SLfree(tmp);
+		  return 0;
+	       }
 	  }
 #endif
 	kill_buffer_cmd(tmp);
@@ -1032,13 +1131,6 @@ int evaluate_cmd() /*{{{*/
 
 /*}}}*/
 
-void insert_string(char *s) /*{{{*/
-{
-   CHECK_READ_ONLY_VOID
-     ins_chars((unsigned char *) s, strlen(s));
-}
-
-/*}}}*/
 
 /* This is weird, Ultrix cc will not compile if set_key comes before unset_key */
 void unset_key(char *key) /*{{{*/
@@ -1063,7 +1155,7 @@ void unset_key_in_keymap(char *key, char *map) /*{{{*/
    
    if (NULL == (kmap = SLang_find_keymap(map)))
      {
-	msg_error(Keymap_Error);
+	jed_verror ("Unknown keymap: %s", map);
 	return;
      }
    
@@ -1085,7 +1177,7 @@ void set_key_in_keymap(char *f, char *key, char *map) /*{{{*/
    
    if (NULL == (kmap = SLang_find_keymap(map)))
      {
-	msg_error(Keymap_Error);
+	jed_verror ("Unknown keymap: %s", map);
 	return;
      }
    
@@ -1321,13 +1413,16 @@ static VFILE *jed_open_lib_file (char *file) /*{{{*/
    lib = Jed_Library;
 
    /* If file begins with ./,then read it from the current directory */
-   if ((*file == '.')
-       && ((file[1] == '/')
+   if ((0 == strncmp (file, "./", 2))
 #ifdef IBMPC_SYSTEM
-	   || (file[1] == '\\')
+       || (0 == strncmp (file, ".\\", 2))
+       || (0 == strncmp (file, "..\\", 3))
 #endif
-       ))
-     lib = "";
+       || (0 == strncmp (file, "../", 3)))
+     {
+	if (NULL == (lib = get_cwd ()))
+	  lib = "";
+     }
    else if ((lib == NULL) || (*lib == 0))
      exit_error("JED_ROOT environment variable needs set.", 0);
 
@@ -1374,9 +1469,9 @@ static VFILE *jed_open_lib_file (char *file) /*{{{*/
 	  }
      }
    
-   if (Batch != 2)
+   if (0 == Jed_Load_Quietly)
      jed_vmessage (1, "loading %s", libf);
-   
+
    return (vp);
 }
 
@@ -1468,7 +1563,19 @@ void load_buffer (void) /*{{{*/
    /* FIXME!!! This should be changed to use the filename if present.  
     * Otherwise, static objects will not get put in the proper table.
     */
-   if (NULL == (x = SLallocate_load_type (cbuf->name)))
+   if (cbuf->file[0] == 0)
+     x = SLallocate_load_type (cbuf->name);
+   else
+     {
+	char *file = jed_dir_file_merge (CBuf->dir, CBuf->file);
+	if (file == NULL)
+	  return;
+	
+	x = SLallocate_load_type (file);
+	SLfree (file);
+     }
+   
+   if (x == NULL)
      return;
 
    x->read = jed_read_from_buffer;
@@ -1494,40 +1601,20 @@ void load_buffer (void) /*{{{*/
    pop_to_buffer (cbuf->name);
    lwant = client_data.l;
 
-   if (lwant != NULL)
+   if (lwant == NULL)
      {
-	bob();
-	while (1)
-	  {
-	     l = CLine->next;
-	     if ((l == NULL) || (l == lwant)) break;
-	     CLine = l; LineNum++;
-	  }
-	(void) skip_whitespace();
+	eob();
+	return;
      }
-}
-
-/*}}}*/
-
-
-void get_key_function() /*{{{*/
-{
-   char *s;
-   int kind;
    
-   s = find_key(&kind);
-   if (s != NULL)
+   bob();
+   while (1)
      {
-	if (SLKeyBoard_Quit && (SLang_Error == USER_BREAK))
-	  {
-	     SLang_Error = 0;
-	     SLKeyBoard_Quit = 0;
-	     /* s = "kbd_quit"; */
-	  }
-	SLang_push_integer(kind);
+	l = CLine->next;
+	if ((l == NULL) || (l == lwant)) break;
+	CLine = l; LineNum++;
      }
-   else s = "";
-   SLang_push_string(s);
+   (void) skip_whitespace();
 }
 
 /*}}}*/
@@ -1628,7 +1715,7 @@ int mini_complete (void) /*{{{*/
 		  set_buffer (Completion_Buffer);
 		  erase_buffer ();
 		  CBuf->flags |= BURIED_BUFFER;
-		  insert_string ("!!! Use Page Up/Down keys to scroll this window. !!!\n");
+		  (void) jed_insert_string ("!!! Use Page Up/Down keys to scroll this window. !!!\n");
 	       }
 	     
 	     n++;
@@ -1660,7 +1747,7 @@ int mini_complete (void) /*{{{*/
 	     if (last_key_char == '\t')
 	       {
 		  while (*pb) pb++;
-		  quick_insert ((unsigned char *)buf, (int) (pb - buf));
+		  (void) jed_quick_insert ((unsigned char *)buf, (int) (pb - buf));
 		  newline ();
 	       }
 	  }
@@ -1700,7 +1787,7 @@ int mini_complete (void) /*{{{*/
      {
 	erase_buffer();
 	/* strcpy(last, buf); */
-	insert_string(last);
+	(void) jed_insert_string(last);
 	if ((n == 1) && ((long) Last_Key_Function == (long) mini_complete))
 	  message("[Sole Completion.]");
      }
@@ -1719,20 +1806,6 @@ int what_char() /*{{{*/
 
 /*}}}*/
 
-void call_cmd(char *str) /*{{{*/
-{
-   SLKeyMap_List_Type *km;
-   int (*fp)(void);
-   
-   km = CBuf->keymap;
-   if (NULL == (fp = (int (*)(void)) (SLang_find_key_function(str, km))))
-     {
-	msg_error("Function does not exist!");
-     }
-   else (void) (*fp)();
-}
-
-/*}}}*/
 
 void copy_region_cmd(char *name) /*{{{*/
 {
@@ -1891,43 +1964,6 @@ void mini_read(char *prompt, char *def, char *stuff) /*{{{*/
 
 /*}}}*/
 
-void get_buffer_info(void) /*{{{*/
-{
-   SLang_push_string(CBuf->file);
-   SLang_push_string(CBuf->dir);
-   SLang_push_string(CBuf->name);
-   SLang_push_integer(CBuf->flags);
-}
-
-/*}}}*/
-
-void set_buffer_info(char *file, char *dir, char *name, int *flags) /*{{{*/
-{
-   char dirbuf [JED_MAX_PATH_LEN + 1];
-   
-   safe_strcpy (dirbuf, dir, sizeof (dirbuf) - 2);
-   fixup_dir (dirbuf);
-   dir = SLang_create_slstring (expand_filename (dirbuf));
-   file = SLang_create_slstring (file);
-   name = SLang_create_slstring (name);
-   
-   if (SLang_Error)
-     {
-	/* SLang_free_slstring ignores NULL */
-	SLang_free_slstring (dir);
-	SLang_free_slstring (file);
-	SLang_free_slstring (name);
-	return;
-     }
-   
-   CBuf->file = file;
-   CBuf->name = name;
-   CBuf->dir = dir;
-   CBuf->flags = *flags;
-}
-
-/*}}}*/
-
 void make_buffer_list(void) /*{{{*/
 {
    int n = 0;
@@ -1997,7 +2033,7 @@ static void jed_traceback(char *s) /*{{{*/
 	n = CBuf->name;
 	set_buffer("*traceback*");
 	eob();
-	insert_string(s);
+	(void) jed_insert_string(s);
 	set_buffer(n);
      }
    else fprintf(stderr, s);
@@ -2139,6 +2175,24 @@ static void vmsg_hook (char *fmt, va_list ap)
    message (buf);
 }
 
+void jed_setup_minibuffer_keymap (void)
+{
+   SLKeyMap_List_Type *m;
+
+   if (NULL == (m = SLang_find_keymap ("Mini_Map")))
+     {
+	if (NULL == (m = SLang_create_keymap("Mini_Map", Global_Map))) 
+	  exit_error("Unable to create minibuffer keymap", 0);
+
+	SLang_undefine_key("\r", m);
+	SLkm_define_key ("\r", (FVOID_STAR) exit_minibuffer, m);
+	SLkm_define_key ("\t", (FVOID_STAR) mini_complete, m);
+	SLkm_define_key (" ", (FVOID_STAR) mini_complete, m);
+     }
+   The_MiniBuffer->keymap = m;
+}
+
+
 void init_minibuffer() /*{{{*/
 {
    Buffer *tmp;
@@ -2172,11 +2226,18 @@ void init_minibuffer() /*{{{*/
        || (-1 == SLang_init_posix_process ())
        || (-1 == SLang_init_posix_dir ())
        || (-1 == SLang_init_slassoc ())
+       || (-1 == SLang_init_stdio ())
+       || (-1 == SLang_init_posix_io ())
+       || (-1 == SLang_init_ospath ())
+       /* || (-1 == SLang_init_import ()) */
        || (-1 == init_jed_intrinsics ())
+       || (-1 == jed_init_userinfo ())
 #if JED_HAS_TTY_MENUS
        || (-1 == jed_init_menus ())
 #endif
+       || (-1 == jed_init_syntax ())
        || (-1 == register_jed_classes ())
+       || (-1 == jed_init_user_hooks ())
        || (-1 == run_init_slang_hooks ()))
      {
 	exit_error("Unable to initialize S-Lang!", 0);
