@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -27,7 +27,7 @@
 #define JED_MARK_TYPE 128
 
 Buffer *Paste_Buffer;
-Buffer *Rectangle_Buffer;
+static Buffer *Rectangle_Buffer;
 
 /* This is used by the narrow command so that multiple windows are 
  * handled properly. The assumption is that we are dealing with a canonical
@@ -95,9 +95,8 @@ static Mark *create_mark (unsigned int flags) /*{{{*/
    Mark *m;
    
    if (NULL == (m = (Mark *) SLmalloc (sizeof(Mark))))
-      {
-	  exit_error("create-mark: malloc error", 0);
-      }
+     exit_error("create-mark: malloc error", 0);
+   
    init_mark (m, flags);
    
    return m;
@@ -360,8 +359,8 @@ static int widen_buffer_lines (Buffer *b) /*{{{*/
      {
 	/* This is a hack to avoid messing with the lock file. Yuk. */
 	unsigned int flags = CBuf->flags;
-	CBuf->flags &= ~READ_ONLY;
-	CBuf->flags |= BUFFER_MODIFIED;
+	CBuf->flags &= ~(READ_ONLY);
+	CBuf->flags |= BUFFER_NON_LOCKING;
 	(void) jed_ins('\n');
 	CBuf->flags = flags;  /* jed_set_buffer_flags (CBuf, flags); */
      }
@@ -382,7 +381,9 @@ static int widen_buffer_lines (Buffer *b) /*{{{*/
    b->nup -= n->nup;
    b->ndown -= n->ndown;
    b->narrow = n->next;
-   
+
+   /* mark_undo_boundary (b); */
+
    SLfree ((char *)n);
    switch_to_buffer(save);
    return(1);
@@ -404,8 +405,11 @@ int widen_buffer (Buffer *b) /*{{{*/
      return widen_buffer_lines (b);
    
    flags = b->flags;
-   b->flags &= ~READ_ONLY;
-   
+   b->flags &= ~(READ_ONLY);
+
+   /* A hack to avoid locking file */
+   b->flags |= BUFFER_NON_LOCKING;
+
    save = CBuf;
    if (b != CBuf) switch_to_buffer (b);
    push_spot ();
@@ -433,6 +437,8 @@ int widen_buffer (Buffer *b) /*{{{*/
 #if JED_HAS_LINE_ATTRIBUTES
    CLine->flags = line_flags;
 #endif
+   /* mark_undo_boundary (b); */
+
    pop_spot ();
    if (save != CBuf) switch_to_buffer (save);
    
@@ -454,10 +460,12 @@ int narrow_to_region (void) /*{{{*/
    if (0 == check_region (&Number_One))/* spot pushed */
      return 0;
 
-   unmark_undo_boundary (CBuf);
+   /* unmark_undo_boundary (CBuf); */
 
    flags = CBuf->flags;
-   CBuf->flags &= ~READ_ONLY;
+   CBuf->flags &= ~(READ_ONLY);
+   /* A hack to avoid locking file */
+   CBuf->flags |= BUFFER_NON_LOCKING;
    
    push_spot ();
    
@@ -479,7 +487,7 @@ int narrow_to_region (void) /*{{{*/
 	(void) jed_insert_newline ();
 	
 #if JED_HAS_LINE_ATTRIBUTES
-	CLine->prev->flags = line_flags;
+	if (CLine->prev != NULL) CLine->prev->flags = line_flags;
 #endif
 	
 	push_mark ();
@@ -507,7 +515,7 @@ int narrow_to_region (void) /*{{{*/
 	jed_insert_newline ();
 #if JED_HAS_LINE_ATTRIBUTES
 	CLine->flags = line_flags;
-	CLine->prev->flags = line_flags;
+	if (CLine->prev != NULL) CLine->prev->flags = line_flags;
 #endif
 
 	push_mark ();
@@ -532,6 +540,7 @@ int narrow_to_region (void) /*{{{*/
 	pop_spot ();
      }
    
+   /* mark_undo_boundary (CBuf); */
    CBuf->flags = flags;
    return 1;
 }
@@ -540,7 +549,7 @@ int narrow_to_region (void) /*{{{*/
 
 int widen (void) /*{{{*/
 {
-   unmark_undo_boundary (CBuf);
+   /* unmark_undo_boundary (CBuf); */
    return widen_buffer_lines (CBuf);
 }
 
@@ -548,7 +557,7 @@ int widen (void) /*{{{*/
 
 int widen_region (void) /*{{{*/
 {
-   unmark_undo_boundary (CBuf);
+   /* unmark_undo_boundary (CBuf); */
    return widen_buffer (CBuf);
 }
 
@@ -567,7 +576,7 @@ int narrow_to_lines () /*{{{*/
    
    if (!check_region(&Number_One)) return(0);       /* spot pushed */
 
-   unmark_undo_boundary (CBuf);
+   /* unmark_undo_boundary (CBuf); */
 
    push_spot();
    pop_mark(&Number_One);
