@@ -1,5 +1,5 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -37,6 +37,12 @@
 # include <windows.h>
 # undef _POSIX_SOURCE
 # include <sys/stat.h>
+# ifdef __BORLANDC__
+#  include <dir.h>
+# endif
+# if defined(__MINGW32__) || defined(__IBMC__)
+#  include <direct.h>
+# endif
 #endif
 
 #include <slang.h>
@@ -58,6 +64,7 @@ int (*X_Init_Term_Hook) (void);
 void (*X_Reset_Term_Hook) (void);
 
 int Ignore_User_Abort = 1;	       /* Abort char triggers S-Lang error */
+char *_Jed_Backspace_Key = "\x7F";
 
 #ifdef IBMPC_SYSTEM
 int Jed_Filename_Case_Sensitive = 0;
@@ -378,22 +385,25 @@ char *extract_file(char *file) /*{{{*/
  */
 int pcsystem_getdisk (void)
 {
-#if defined(__MINGW32__)
+#if defined(__MINGW32__) || defined(__IBMC__)
    return _getdrive ();
 #else
 # if defined(__DJGPP__)
    int d;  _dos_getdrive (&d); return d;
 # else
+#  if defined(__BORLANDC__)
+   return 1 + getdisk();
+#  else
    /* This may not work */
    char *cwd = get_cwd ();
-   char c;
 
-   if ((cwd == NULL) || (*cwd == 0) || (c[1] != ':'))
+   if ((cwd == NULL) || (*cwd == 0) || (cwd[1] != ':'))
      cwd = "C:";
 
    return 1 + (UPPER_CASE (*cwd) - 'A');
-# endif
-#endif				       /* MINGW32 */
+#  endif			       /* __BORLANDC__ */
+# endif				       /* __DJGPP__ */
+#endif				       /* __MINGW32__ */
 }
 #endif				       /* IBMPC_SYSTEM */
 
